<?php
/**
 * Copyright 2021 PayXpert
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Payxpert\Connect2Pay\Model\Payment;

use Magento\Framework\Exception\PaymentException;
use Magento\Framework\Validator\Exception;
use Magento\Variable\Model\VariableFactory;
use Magento\Framework\Model\Context;
use Magento\Framework\Registry;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Payment\Helper\Data as PaymentHelper;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Payment\Model\Method\Logger;
use Magento\Framework\UrlInterface;
use Magento\Checkout\Model\Session;
use Magento\Sales\Model\Order;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\Escaper;
use Payxpert\Connect2Pay\Helper\Data as PayxpertHelper;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Data\Collection\AbstractDb;

use Magento\Payment\Model\Method\AbstractMethod;
use PayXpert\Connect2Pay\Connect2PayClient;

class Payxpert extends AbstractMethod
{
    const PAYMENT_METHOD_CODE = 'payxpert';

    protected $_code = self::PAYMENT_METHOD_CODE;
    protected $_isGateway = true;
    protected $_canRefund = true;
    protected $_canRefundInvoicePartial = true;

    protected $checkoutSession;
    protected $order;
    protected $urlBuilder;
    protected $customerSession;
    protected $escaper;
    protected $helper;
    protected $variable;

    /**
     * PayXpert constructor.
     *
     * @param VariableFactory $_variable
     * @param Context $context
     * @param Registry $registry
     * @param ExtensionAttributesFactory $extensionFactory
     * @param AttributeValueFactory $customAttributeFactory
     * @param PaymentHelper $paymentData
     * @param ScopeConfigInterface $scopeConfig
     * @param Logger $logger
     * @param UrlInterface $urlBuilder
     * @param Session $checkoutSession
     * @param Order $order
     * @param CustomerSession $customerSession
     * @param Escaper $escaper
     * @param PayxpertHelper $helper
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        VariableFactory $_variable,
        Context $context,
        Registry $registry,
        ExtensionAttributesFactory $extensionFactory,
        AttributeValueFactory $customAttributeFactory,
        PaymentHelper $paymentData,
        ScopeConfigInterface $scopeConfig,
        Logger $logger,
        UrlInterface $urlBuilder,
        Session $checkoutSession,
        Order $order,
        CustomerSession $customerSession,
        Escaper $escaper,
        PayxpertHelper $helper,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );
        $this->variable = $_variable;
        $this->urlBuilder = $urlBuilder;
        $this->checkoutSession = $checkoutSession;
        $this->order = $order;
        $this->customerSession = $customerSession;
        $this->escaper = $escaper;
        $this->helper = $helper;
        $this->logger = $logger;
    }

    /**
     * @param Order $order
     * @param string $paymentMethod
     * @param false $paymentNetwork
     * @return string
     * @throws Exception
     * @throws \Exception
     */
    public function startTransaction(
        Order $order,
        $paymentMethod = "CreditCard",
        $paymentNetwork = false
    ): string {
        $order->getCustomerFirstname();

        $originator = $this->helper->getConfig('payment/payxpert/originator');
        $password = $this->helper->getConfig('payment/payxpert/password');

        $c2pClient = new Connect2PayClient($this->getUrl(), $originator, $password);

        $billingAddress = $order->getBillingAddress();
        $shippingAddress = $order->getShippingAddress();

        $c2pClient->setOrderID($order->getId());
        // $c2pClient->setCustomerIP( $fields['customerIP'] );
        $c2pClient->setPaymentMethod($paymentMethod);
        if ($paymentNetwork) {
            $c2pClient->setPaymentNetwork($paymentNetwork);
        }

        $c2pClient->setPaymentMode(Connect2PayClient::PAYMENT_MODE_SINGLE);
        $c2pClient->setShopperID($order->getCustomerID());
        $c2pClient->setShippingType(Connect2PayClient::SHIPPING_TYPE_VIRTUAL);
        $c2pClient->setAmount($order->getGrandTotal() * 100);
        $c2pClient->setOrderDescription("Order #" . $order->getId());
        $c2pClient->setCurrency($order->getOrderCurrency()->getCurrencyCode());

        $c2pClient->setShopperFirstName($order->getCustomerFirstname());
        $c2pClient->setShopperLastName($order->getCustomerLastname());
        $c2pClient->setShopperAddress($billingAddress->getStreetLine(1));
        $c2pClient->setShopperZipcode($billingAddress->getPostcode());
        $c2pClient->setShopperCity($billingAddress->getCity());
        $c2pClient->setShopperState($billingAddress->getRegion());
        $c2pClient->setShopperCountryCode($billingAddress->getCountryId());
        $c2pClient->setShopperPhone($billingAddress->getTelephone());
        $c2pClient->setShopperEmail($order->getCustomerEmail() ?: $billingAddress->getEmail());

        if ($shippingAddress) {
            $c2pClient->setShipToFirstName($shippingAddress->getFirstname());
            $c2pClient->setShipToLastName($shippingAddress->getLastname());
            $c2pClient->setShipToAddress($shippingAddress->getStreetLine(1));
            $c2pClient->setShipToZipcode($shippingAddress->getPostcode());
            $c2pClient->setShipToCity($shippingAddress->getCity());
            $c2pClient->setShipToState($shippingAddress->getRegion());
            $c2pClient->setShipToCountryCode($shippingAddress->getCountryId());
            $c2pClient->setShipToPhone($shippingAddress->getTelephone());
        }

        $c2pClient->setCtrlRedirectURL($this->urlBuilder->getUrl('payxpert/checkout/success/'));
        $c2pClient->setCtrlCallbackURL($this->urlBuilder->getUrl('payxpert/checkout/callback/'));

        $c2pClient->setCtrlCustomData($this->checkoutSession->getSessionId());

        if ($c2pClient->validate()) {
            if ($c2pClient->preparePayment()) {

                $this->customerSession->setMerchantToken($c2pClient->getMerchantToken());
                $customerToken = $c2pClient->getCustomerToken();
                $merchantToken = $c2pClient->getMerchantToken();

                $variable = $this->variable->create();
                $data = [
                    'code' => $customerToken,
                    'name' => 'Merchant token from customer token',
                    'html_value' => '',
                    'plain_value' => $merchantToken,

                ];
                $variable->setData($data);
                try {
                    $variable->save();
                } catch (\Exception $e) {
                    $message = $e->getMessage();
                    $this->logger->debug([$message]);
                }

                $paymentUrl = $c2pClient->getCustomerRedirectURL();
                $order->setStatus('pending_payment');
                $order->save();
            } else {
                $message = "Preparation error at " . $c2pClient->getURL() . " occurred: " .
                    $this->escaper->escapeHtml($c2pClient->getClientErrorMessage());
                throw new PaymentException(__($message));

            }
        } else {
            $message = "Validation error occurred: " . $this->escaper->escapeHtml($c2pClient->getClientErrorMessage());
            throw new PaymentException(__($message));
        }

        return $paymentUrl;
    }

    /**
     * @return string
     */
    public function getUrl(): string
    {
        $url = trim($this->helper->getConfig('payment/payxpert/url'));
        if (empty($url)) {
            $url = "https://connect2.payxpert.com";
        }
        return $url;
    }
}
